#!/usr/bin/python3
#
# Copyright 2015 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the example portserver."""

import asyncio
import os
import socket
import sys
import unittest
from unittest import mock

import portpicker
import portserver



def setUpModule():
  portserver._configure_logging(verbose=True)


class PortserverFunctionsTest(unittest.TestCase):

  @classmethod
  def setUp(cls):
    cls.port = portpicker.PickUnusedPort()

  def test_get_process_command_line(self):
    portserver._get_process_command_line(os.getpid())

  def test_get_process_start_time(self):
    self.assertGreater(portserver._get_process_start_time(os.getpid()), 0)

  def test_port_is_available_true(self):
    """This might be flaky unless this test is run with a portserver."""
    # Insert Inception "we must go deeper" meme here.
    self.assertTrue(portserver._port_is_available(self.port))

  def test_port_is_available_false(self):
    with mock.patch.object(socket, 'socket') as mock_sock:
      mock_sock.side_effect = socket.error('fake socket error', 0)
      self.assertFalse(portserver._port_is_available(self.port))

  def test_should_allocate_port(self):
    self.assertFalse(portserver._should_allocate_port(0))
    self.assertFalse(portserver._should_allocate_port(1))
    self.assertTrue(portserver._should_allocate_port, os.getpid())
    child_pid = os.fork()
    if child_pid == 0:
      os._exit(0)
    else:
      os.waitpid(child_pid, 0)
    # This test assumes that after waitpid returns the kernel has finished
    # cleaning the process.  We also assume that the kernel will not reuse
    # the former child's pid before our next call checks for its existence.
    # Likely assumptions, but not guaranteed.
    self.assertFalse(portserver._should_allocate_port(child_pid))

  def test_parse_command_line(self):
    with mock.patch.object(
        sys, 'argv', ['program_name', '--verbose',
                      '--portserver_static_pool=1-1,3-8',
                      '--portserver_unix_socket_address=@hello-test']):
      portserver._parse_command_line()

  def test_parse_port_ranges(self):
    self.assertFalse(portserver._parse_port_ranges(''))
    self.assertCountEqual(portserver._parse_port_ranges('1-1'), {1})
    self.assertCountEqual(portserver._parse_port_ranges('1-1,3-8,375-378'),
                          {1, 3, 4, 5, 6, 7, 8, 375, 376, 377, 378})
    # Unparsable parts are logged but ignored.
    self.assertEqual({1, 2}, portserver._parse_port_ranges('1-2,not,numbers'))
    self.assertEqual(set(), portserver._parse_port_ranges('8080-8081x'))
    # Port ranges that go out of bounds are logged but ignored.
    self.assertEqual(set(), portserver._parse_port_ranges('0-1138'))
    self.assertEqual(set(range(19, 84 + 1)),
                     portserver._parse_port_ranges('1138-65536,19-84'))

  def test_configure_logging(self):
    """Just code coverage really."""
    portserver._configure_logging(False)
    portserver._configure_logging(True)

  @mock.patch.object(sys, 'argv',
                     ['PortserverFunctionsTest.test_main',
                      '--portserver_unix_socket_address=@TST-%d' % os.getpid()])
  @mock.patch.object(portserver, '_parse_port_ranges')
  @mock.patch('asyncio.get_event_loop')
  @mock.patch('asyncio.start_unix_server')
  def test_main(self, *unused_mocks):
    portserver._parse_port_ranges.return_value = set()
    with self.assertRaises(SystemExit):
      portserver.main()

    # Give it at least one port and try again.
    portserver._parse_port_ranges.return_value = {self.port}

    mock_event_loop = mock.Mock(spec=asyncio.base_events.BaseEventLoop)
    asyncio.get_event_loop.return_value = mock_event_loop
    asyncio.start_unix_server.return_value = mock.Mock()
    mock_event_loop.run_forever.side_effect = KeyboardInterrupt

    portserver.main()

    mock_event_loop.run_until_complete.assert_any_call(
        asyncio.start_unix_server.return_value)
    mock_event_loop.close.assert_called_once_with()
    # NOTE: This could be improved.  Tests of main() are often gross.


class PortPoolTest(unittest.TestCase):

  @classmethod
  def setUpClass(cls):
    cls.port = portpicker.PickUnusedPort()

  def setUp(self):
    self.pool = portserver._PortPool()

  def test_initialization(self):
    self.assertEqual(0, self.pool.num_ports())
    self.pool.add_port_to_free_pool(self.port)
    self.assertEqual(1, self.pool.num_ports())
    self.pool.add_port_to_free_pool(1138)
    self.assertEqual(2, self.pool.num_ports())
    self.assertRaises(ValueError, self.pool.add_port_to_free_pool, 0)
    self.assertRaises(ValueError, self.pool.add_port_to_free_pool, 65536)

  @mock.patch.object(portserver, '_port_is_available')
  def test_get_port_for_process_ok(self, mock_port_is_available):
    self.pool.add_port_to_free_pool(self.port)
    mock_port_is_available.return_value = True
    self.assertEqual(self.port, self.pool.get_port_for_process(os.getpid()))
    self.assertEqual(1, self.pool.ports_checked_for_last_request)

  @mock.patch.object(portserver, '_port_is_available')
  def test_get_port_for_process_none_left(self, mock_port_is_available):
    self.pool.add_port_to_free_pool(self.port)
    self.pool.add_port_to_free_pool(22)
    mock_port_is_available.return_value = False
    self.assertEqual(2, self.pool.num_ports())
    self.assertEqual(0, self.pool.get_port_for_process(os.getpid()))
    self.assertEqual(2, self.pool.num_ports())
    self.assertEqual(2, self.pool.ports_checked_for_last_request)


@mock.patch.object(portserver, '_get_process_command_line')
@mock.patch.object(portserver, '_should_allocate_port')
@mock.patch.object(portserver._PortPool, 'get_port_for_process')
class PortServerRequestHandlerTest(unittest.TestCase):

  def setUp(self):
    portserver._configure_logging(verbose=True)
    self.rh = portserver._PortServerRequestHandler([23, 42, 54])

  def test_stats_reporting(self, *unused_mocks):
    with mock.patch.object(portserver, 'log') as mock_logger:
      self.rh.dump_stats()
    mock_logger.info.assert_called_with('total-allocations 0')

  def test_handle_port_request_bad_data(self, *unused_mocks):
    self._test_bad_data_from_client(b'')
    self._test_bad_data_from_client(b'\n')
    self._test_bad_data_from_client(b'99Z\n')
    self._test_bad_data_from_client(b'99 8\n')
    self.assertEqual([], portserver._get_process_command_line.mock_calls)

  def _test_bad_data_from_client(self, data):
    mock_writer = mock.Mock(asyncio.StreamWriter)
    self.rh._handle_port_request(data, mock_writer)
    self.assertFalse(portserver._should_allocate_port.mock_calls)

  def test_handle_port_request_denied_allocation(self, *unused_mocks):
    portserver._should_allocate_port.return_value = False
    self.assertEqual(0, self.rh._denied_allocations)
    mock_writer = mock.Mock(asyncio.StreamWriter)
    self.rh._handle_port_request(b'5\n', mock_writer)
    self.assertEqual(1, self.rh._denied_allocations)

  def test_handle_port_request_bad_port_returned(self, *unused_mocks):
    portserver._should_allocate_port.return_value = True
    self.rh._port_pool.get_port_for_process.return_value = 0
    mock_writer = mock.Mock(asyncio.StreamWriter)
    self.rh._handle_port_request(b'6\n', mock_writer)
    self.rh._port_pool.get_port_for_process.assert_called_once_with(6)
    self.assertEqual(1, self.rh._denied_allocations)

  def test_handle_port_request_success(self, *unused_mocks):
    portserver._should_allocate_port.return_value = True
    self.rh._port_pool.get_port_for_process.return_value = 999
    mock_writer = mock.Mock(asyncio.StreamWriter)
    self.assertEqual(0, self.rh._total_allocations)
    self.rh._handle_port_request(b'8', mock_writer)
    portserver._should_allocate_port.assert_called_once_with(8)
    self.rh._port_pool.get_port_for_process.assert_called_once_with(8)
    self.assertEqual(1, self.rh._total_allocations)
    self.assertEqual(0, self.rh._denied_allocations)
    mock_writer.write.assert_called_once_with(b'999\n')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python3
#
# Copyright 2007 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Pure python code for finding unused ports on a host.

This module provides a pure python implementation of PickUnusedPort.
It can also be called via the command line for use in shell scripts.
When called from the command line, it takes one optional argument, which,
if given, is sent to portserver instead of portpicker's PID.
To reserve a port for the lifetime of a bash script, use $BASHPID as this
argument.

There is a race condition between picking a port and your application code
binding to it.  The use of a port server to prevent that is recommended on
loaded test hosts running many tests at a time.

If your code can accept a bound socket as input rather than being handed a
port number consider using socket.bind(('localhost', 0)) to bind to an
available port without a race condition rather than using this library.

Typical usage:
  test_port = portpicker.PickUnusedPort()
"""

from __future__ import print_function
import os
import random
import socket
import sys


_PROTOS = [(socket.SOCK_STREAM, socket.IPPROTO_TCP),
           (socket.SOCK_DGRAM, socket.IPPROTO_UDP)]


def Bind(port, socket_type, socket_proto):
  """Try to bind to a socket of the specified type, protocol, and port.

  This is primarily a helper function for PickUnusedPort, used to see
  if a particular port number is available.

  Args:
    port: The port number to bind to, or 0 to have the OS pick a free port.
    socket_type: The type of the socket (ex: socket.SOCK_STREAM).
    socket_proto: The protocol of the socket (ex: socket.IPPROTO_TCP).

  Returns:
    The port number on success or None on failure.
  """
  s = socket.socket(socket.AF_INET, socket_type, socket_proto)
  try:
    try:
      s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
      s.bind(('', port))
      return s.getsockname()[1]
    except socket.error:
      return None
  finally:
    s.close()


def IsPortFree(port):
  """Check if specified port is free.

  Args:
    port: integer, port to check
  Returns:
    boolean, whether it is free to use for both TCP and UDP
  """
  return (Bind(port, _PROTOS[0][0], _PROTOS[0][1]) and
          Bind(port, _PROTOS[1][0], _PROTOS[1][1]))


def PickUnusedPort(pid=None):
  """A pure python implementation of PickUnusedPort.

  Args:
    pid: PID to tell the portserver to associate the reservation with. If None,
      the current process's PID is used.

  Returns:
    A port number that is unused on both TCP and UDP.
  """
  port = None
  # Provide access to the portserver on an opt-in basis.
  if 'PORTSERVER_ADDRESS' in os.environ:
    port = _GetPortFromPortServer(os.environ['PORTSERVER_ADDRESS'], pid=pid)
  if not port:
    return _PickUnusedPortWithoutServer()
  return port


def _PickUnusedPortWithoutServer():
  """A pure python implementation of PickUnusedPort_NoServer().

  This code ensures that the port is available on both TCP and UDP.

  This function is an implementation detail of PickUnusedPort(), and
  should not be called by code outside of this module.

  Returns:
    A port number that is unused on both TCP and UDP.  None on error.
  """
  # Try random ports first.
  r = random.Random()
  for _ in range(10):
    port = int(r.randrange(32768, 60000))
    if IsPortFree(port):
      return port

  # Try OS-assigned ports next.
  # Ambrose discovered that on the 2.6 kernel, calling Bind() on UDP socket
  # returns the same port over and over. So always try TCP first.
  while True:
    # Ask the OS for an unused port.
    port = Bind(0, _PROTOS[0][0], _PROTOS[0][1])
    # Check if this port is unused on the other protocol.
    if port and Bind(port, _PROTOS[1][0], _PROTOS[1][1]):
      return port


def _GetPortFromPortServer(portserver_address, pid=None):
  """Request a free a port from a system-wide portserver.

  This follows a very simple portserver protocol:
  The request consists of our pid (in ASCII) followed by a newline.
  The response is a port number and a newline, 0 on failure.

  This function is an implementation detail of PickUnusedPort(), and
  should not be called by code outside of this module.

  Args:
    portserver_address: The address (path) of a unix domain socket
      with which to connect to the portserver.  A leading '@'
      character indicates an address in the "abstract namespace."
    pid: The PID to tell the portserver to associate the reservation with.
      If None, the current process's PID is used.

  Returns:
    The port number on success or None on failure.
  """
  if not portserver_address:
    return None
  # An AF_UNIX address may start with a zero byte, in which case it is in the
  # "abstract namespace", and doesn't have any filesystem representation.
  # See 'man 7 unix' for details.
  # The convention is to write '@' in the address to represent this zero byte.
  if portserver_address[0] == '@':
    portserver_address = '\0' + portserver_address[1:]

  if pid is None:
    pid = os.getpid()

  try:
    # Create socket.
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    try:
      # Connect to portserver.
      sock.connect(portserver_address)

      # Write request.
      sock.sendall(('%d\n' % pid).encode('ascii'))

      # Read response.
      # 1K should be ample buffer space.
      buf = sock.recv(1024)
    finally:
      sock.close()
  except socket.error:
    print ('Socket error when connecting to portserver.', file=sys.stderr)
    return None

  try:
    return int(buf.split(b'\n')[0])
  except ValueError:
    print ('Portserver failed to find a port.', file=sys.stderr)
    return None


if __name__ == '__main__':
  # If passed an argument, cast it to int and treat it as a PID, otherwise pass
  # pid=None to use portpicker's PID.
  port = PickUnusedPort(pid=int(sys.argv[1]) if len(sys.argv) > 1 else None)
  if not port:
    sys.exit(1)
  print (port)



"""Simple distutils setup for the pure Python portpicker."""

import sys
import textwrap
import distutils.core


def main():
  requires = []
  scripts = []
  py_version = sys.version_info[:2]
  if py_version < (3, 3):
    requires.append('mock(>=1.0)')
  if py_version == (3, 3):
    requires.append('asyncio(>=3.4)')
  if py_version >= (3, 3):
    # The example portserver implementation requires Python 3 and asyncio.
    scripts.append('src/portserver.py')

  distutils.core.setup(
      name='portpicker',
      version='1.0.0',
      description='A library to choose unique available network ports.',
      long_description=textwrap.dedent("""
          Portpicker provides an API to find and return an available network
          port for an application to bind to.  Ideally suited for use from
          unittests or for test harnesses that launch a local server."""),
      license='Apache 2.0',
      maintainer='Google',
      url='https://github.com/google/python_portpicker',
      package_dir={'': 'src'},
      py_modules=['portpicker'],
      platforms=['POSIX'],
      requires=requires,
      scripts=scripts,
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: Apache Software License',
          'Intended Audience :: Developers',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: Jython',
          'Programming Language :: Python :: Implementation :: PyPy']
  )


if __name__ == '__main__':
  main()



#!/usr/bin/python
#
# Copyright 2007 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittests for the portpicker module."""

import os
import random
import socket
import unittest

try:
  # pylint: disable=no-name-in-module
  from unittest import mock  # Python >= 3.3.
except ImportError:
  import mock  # https://pypi.python.org/pypi/mock

import portpicker


class PickUnusedPortTest(unittest.TestCase):

  def IsUnusedTCPPort(self, port):
    return self._bind(port, socket.SOCK_STREAM, socket.IPPROTO_TCP)

  def IsUnusedUDPPort(self, port):
    return self._bind(port, socket.SOCK_DGRAM, socket.IPPROTO_UDP)

  def setUp(self):
    # So we can Bind even if portpicker.Bind is stubbed out.
    self._bind = portpicker.Bind

  def testPickUnusedPortActuallyWorks(self):
    """This test can be flaky."""
    for _ in range(10):
      port = portpicker.PickUnusedPort()
      self.assertTrue(self.IsUnusedTCPPort(port))
      self.assertTrue(self.IsUnusedUDPPort(port))

  @unittest.skipIf('PORTSERVER_ADDRESS' not in os.environ,
                   'no port server to test against')
  def testPickUnusedCanSuccessfullyUsePortServer(self):

    with mock.patch.object(portpicker, '_PickUnusedPortWithoutServer'):
      portpicker._PickUnusedPortWithoutServer.side_effect = Exception('eek!')

      # Since _PickUnusedPortWithoutServer() raises an exception, if we
      # can successfully obtain a port, the portserver must be working.
      port = portpicker.PickUnusedPort()
      self.assertTrue(self.IsUnusedTCPPort(port))
      self.assertTrue(self.IsUnusedUDPPort(port))

  @unittest.skipIf('PORTSERVER_ADDRESS' not in os.environ,
                   'no port server to test against')
  def testGetPortFromPortServer(self):
    """Exercise the _GetPortFromPortServer() helper function."""
    for _ in range(10):
      port = portpicker._GetPortFromPortServer(os.environ['PORTSERVER_ADDRESS'])
      self.assertTrue(self.IsUnusedTCPPort(port))
      self.assertTrue(self.IsUnusedUDPPort(port))

  def testSendsPidToPortServer(self):
    server = mock.Mock()
    server.recv.return_value = b'42768\n'
    with mock.patch.object(socket, 'socket', return_value=server):
      port = portpicker._GetPortFromPortServer('portserver', pid=1234)
      server.sendall.assert_called_once_with(b'1234\n')
    self.assertEqual(port, 42768)

  def testPidDefaultsToOwnPid(self):
    server = mock.Mock()
    server.recv.return_value = b'52768\n'
    with mock.patch.object(socket, 'socket', return_value=server):
      with mock.patch.object(os, 'getpid', return_value=9876):
        port = portpicker._GetPortFromPortServer('portserver')
        server.sendall.assert_called_once_with(b'9876\n')
    self.assertEqual(port, 52768)

  def testRandomlyChosenPorts(self):
    # Unless this box is under an overwhelming socket load, this test
    # will heavily exercise the "pick a port randomly" part of the
    # port picking code, but may never hit the "OS assigns a port"
    # code.
    for _ in range(100):
      port = portpicker._PickUnusedPortWithoutServer()
      self.assertTrue(self.IsUnusedTCPPort(port))
      self.assertTrue(self.IsUnusedUDPPort(port))

  def testOSAssignedPorts(self):
    self.last_assigned_port = None

    def ErrorForExplicitPorts(port, socket_type, socket_proto):
      # Only successfully return a port if an OS-assigned port is
      # requested, or if we're checking that the last OS-assigned port
      # is unused on the other protocol.
      if port == 0 or port == self.last_assigned_port:
        self.last_assigned_port = self._bind(port, socket_type, socket_proto)
        return self.last_assigned_port
      else:
        return None

    with mock.patch.object(portpicker, 'Bind', ErrorForExplicitPorts):
      for _ in range(100):
        port = portpicker._PickUnusedPortWithoutServer()
        self.assertTrue(self.IsUnusedTCPPort(port))
        self.assertTrue(self.IsUnusedUDPPort(port))

  def testPickPortsWithError(self):
    r = random.Random()

    def BindWithError(port, socket_type, socket_proto):
      # 95% failure rate means both port picking methods will be exercised.
      if int(r.uniform(0, 20)) == 0:
        return self._bind(port, socket_type, socket_proto)
      else:
        return None

    with mock.patch.object(portpicker, 'Bind', BindWithError):
      for _ in range(100):
        port = portpicker._PickUnusedPortWithoutServer()
        self.assertTrue(self.IsUnusedTCPPort(port))
        self.assertTrue(self.IsUnusedUDPPort(port))


if __name__ == '__main__':
  unittest.main()
